<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 *
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\Categories;

use Webkul\SalesforceConnector\Helper\Category;

class Catsync extends \Magento\Backend\App\Action
{
    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $_resultPageFactory;
    // protected $_categoryCollection;
    protected $_filter;
    protected $_helper;
    protected $_helperData;
    protected $_helperConnection;
    /**
     * @var CollectionFactory
     */
    protected $_collectionFactory;

    /**
     * @param \Magento\Backend\App\Action\Context        $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Ui\Component\MassAction\Filter $filter,
        \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $collectionFactory,
        Category $helper,
        \Webkul\SalesforceConnector\Helper\Data $data
    ) {
        parent::__construct($context);
        $this->_resultPageFactory = $resultPageFactory;

        $this->_helper = $helper;
        $this->_helperData = $data;
        $this->_collectionFactory = $collectionFactory;
        $this->_filter = $filter;
    }

    public function execute()
    {
        $syncEnable = $this->_helperData->getConfigValue('catPdt', 'synCat');
        $conn_status = \Webkul\SalesforceConnector\Helper\Connection::checkConnectionStatus();
        if (!$conn_status) {
            $this->messageManager
                ->addError(__('Stablish Connection first'));
        } elseif ($syncEnable) {
            $id = $this->getRequest()->getParam('entity_id');
            if ($id) {
                $sfId = $this->_helper->exportSpecificCategory($id);
                if ($sfId) {
                    $this->messageManager->addSuccess(__('Category has been successfully Exported at Salesforce.', $sfId));
                }
            } else {
                $ids = $this->getRequest()->getParam('selected');
                $success=0;
                if ($ids) {
                    $ids=array_chunk($ids,4);
                    foreach($ids as $id){
                        $exportData=$this->_helper->exportCategories($id, true);
                        $success+=$exportData['success'];
                    }                              
                    $this->messageManager->addSuccess(
                        __('A total of %1 record(s) have been processed.', $success)
                    );
                } else {
                    $excludedIds = $this->getRequest()->getParam('excluded');
                    $collection = $this->_collectionFactory->create();
                    $allCategoryIds = array();
                    foreach ($collection->getItems() as $item) {
                        array_push($allCategoryIds, $item->getID());
                    }
                    if ($excludedIds == 'false') {
                        $allCategoryIds=array_chunk($allCategoryIds,4);
                        foreach($allCategoryIds as $id){
                            $exportData=$this->_helper->exportCategories($id, true);
                            $success+=$exportData['success'];    
                        }      
                        $this->messageManager->addSuccess(
                            __('A total of %1 record(s) have been processed.', $success)
                        );
                    } else {
                        $ids = array_diff($allCategoryIds, $excludedIds);
                        $ids=array_chunk($ids,4);
                        foreach($ids as $id){
                            $exportData=$this->_helper->exportCategories($id, true);
                            $success+=$exportData['success'];    
                        }  
                        $this->messageManager->addSuccess(
                            __('A total of %1 record(s) have been processed.', $success)
                        );
                    }
                }
            }
            //else {
            // in case sellect all or exclude some
            //TODO FIX: we can do only this block of code but
            //this will first fetch magento main category data
            //then passed to export so to reduce db call.
            // we got magento id
            //check for excluded
            // $excluded = $this->getRequest()
            //               ->getParam('excluded');
            // // we got magento id
            // $categoryCollection = $this->_objectManager
            //                            ->create('Magento\Catalog\Model\ResourceModel\Category\Collection');
            // if (is_array($excluded) && !empty($excluded)) {
            //     $categoryCollection->addFieldToFilter(
            //        'entity_id',
            //        ['nin' => $excluded]
            //);
            // }
            // $collection = $this->_filter->getCollection($categoryCollection);
            // $recordProcessed = 0;
            // foreach ($collection->getItems() as $item) {
            //     $sfId = $this->_helper->exportSpecificCategory($item->getId());
            //     ++$recordProcessed;
            // }
            // $this->messageManager->addSuccess(
            //     __('A total of %1 record(s) have been processed.', $recordProcessed)
            // );
            //}
        } else {
            $this->messageManager
                ->addError(__('Category sync is not enabled.Please enable and then sync'));
        }
        $resultRedirect = $this->resultRedirectFactory->create();
        $resultRedirect->setPath('sfconnector/categories/index/');

        return $resultRedirect;
    }
}
