<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 * @package   Webkul_Eshopsync
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\Categories;

use \Magento\Backend\App\Action\Context;
use \Magento\Framework\View\Result\PageFactory;

class GetAllUpdateCategory extends \Magento\Backend\App\Action
{
    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $_resultPageFactory = false;

    /** @var \Magento\Framework\Json\Helper\Data */
    protected $_json;

    /** @var \Webkul\Eshopsync\Model\CategoryFactory */
    protected $_categoryModel;

    /** @var \Magento\Catalog\Model\CategoryFactory */
    protected $_catalogCategoryModel;
    protected $_helperData;

    /**
     * @param \Magento\Backend\App\Action\Context
     * @param \Webkul\Eshopsync\Helper\Connection $connection
     * @param \Magento\Framework\Json\Helper\Data $jsonHelper
     * @param \Magento\Catalog\Model\CategoryFactory $catalogCategory
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Webkul\SalesforceConnector\Model\CategoryFactory $category,
        \Magento\Catalog\Model\CategoryFactory $catalogCategory,
        \Webkul\SalesforceConnector\Helper\Data $data,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory
    ) {

        parent::__construct($context);
        $this->_resultPageFactory = $resultPageFactory;
        $this->_json = $jsonHelper;
        $this->_categoryModel = $category;
        $this->_helperData = $data;
        $this->_catalogCategoryModel = $catalogCategory;
    }

    public function execute()
    {
        $exportIds = [];
        $syncEnable = $this->_helperData->getConfigValue('catPdt', 'synCat');
        if (\Webkul\SalesforceConnector\Helper\Connection::checkConnectionStatus()) {
            if ($syncEnable) {
                $categoryCollection = $this->_categoryModel->create()->getCollection()
                    ->addFieldToSelect('magento_id')
                    ->addFieldToFilter('sync_status', '1')->getData();
                if (count($categoryCollection) == 0) {
                    $this->messageManager->addError(__("Update requires at least one category exported/synced with Salesforce."));
                } else {
                    $exportIds = array_column($categoryCollection, 'magento_id');
                    if (count($exportIds) == 0) {
                        $this->messageManager->addNotice(__("All Magento Categories are already updated at salesforce."));
                    }
                }
            } else {
                $this->messageManager->addError(__("Check Setting."));
            }
        } else {
            $this->messageManager->addError(__("Check Connection."));
        }
        $response = ['categ_ids' => $exportIds];
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($this->_json->jsonEncode($response));
    }

    /*
     * Check permission via ACL resource
     */
    // protected function _isAllowed()
    // {
    //     return true;
    // }
}
