<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 * @package   Webkul_Eshopsync
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Helper;

use \Magento\Framework\App\Helper\AbstractHelper;

// use Webkul\SalesforceConnector\Logger\EshopsyncLogger as Logger;

/**
 * Eshopsync customer helper.
 */

class Customer extends AbstractHelper
{
    protected $_json;
    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    protected $_customerCustomer;

    /**
     * @var \Magento\Customer\Model\AddressFactory
     */
    protected $_customerAddress;

    /**
     * @var \Webkul\Eshopsync\Model\CustomerFactory
     */
    protected $_eshopsyncCustomer;

    /**
     * @var \Webkul\Eshopsync\Model\ContactFactory
     */
    protected $_eshopsyncContact;

    /**
     * @var \Webkul\Eshopsync\Helper\Data
     */
    protected $_helperData;

    /**
     * @var \Webkul\Eshopsync\Helper\Connection
     */
    protected $_helperConnection;

    /**
     * @var \Webkul\Eshopsync\Helper\Contact
     */
    // protected $_helperContact;

    /**
     * @var Logger
     */
    // protected $_logger;

    /**
     * @param Magento\Framework\App\Helper\Context    $context
     * @param \Magento\Customer\Model\CustomerFactory $customerCustomer
     * @param \Magento\Customer\Model\AddressFactory  $customerAddress
     * @param \Webkul\Eshopsync\Model\CustomerFactory $eshopsyncCustomer
     * @param \Webkul\Eshopsync\Model\ContactFactory  $eshopsyncContact
     * @param \Webkul\Eshopsync\Helper\Data           $helperData
     * @param \Webkul\Eshopsync\Helper\Connection     $helperConnection
     * @param \Webkul\Eshopsync\Helper\Contact        $helperContact
     * @param Logger                                  $logger
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Customer\Model\CustomerFactory $customerCustomer,
        \Magento\Customer\Model\AddressFactory $customerAddress,
        \Webkul\SalesforceConnector\Model\CustomerFactory $eshopsyncCustomer,
        \Webkul\SalesforceConnector\Model\ContactFactory $eshopsyncContact,
        \Webkul\SalesforceConnector\Helper\Data $helperData,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Webkul\SalesforceConnector\Helper\Connection $helperConnection
        // \Webkul\SalesforceConnector\Helper\Contact $helperContact
        // Logger $logger
    ) {
        $this->_customerCustomer = $customerCustomer;
        $this->_customerAddress = $customerAddress;
        $this->_eshopsyncCustomer = $eshopsyncCustomer;
        $this->_eshopsyncContact = $eshopsyncContact;
        $this->_helperData = $helperData;
        $this->_helperConnection = $helperConnection;
        $this->_json = $jsonHelper;
        // $this->_helperContact = $helperContact;
        // $this->_logger = $logger;
        parent::__construct($context);
    }
    public function customerMapping($data, $error = 0)
    {
        $model = $this->_eshopsyncCustomer->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $data['magento_id'])->getData();
        if (count($collection) == 1) {
            $data['entity_id'] = $collection[0]['entity_id'];
        } elseif (count($collection) > 1) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
        $model->setData($data);
        $model->save();
    }
    public function contactMapping($data, $error = 0)
    {
        $model = $this->_eshopsyncContact->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $data['magento_id'])->getData();
        if (count($collection) == 1) {
            $data['entity_id'] = $collection[0]['entity_id'];
        } elseif (count($collection) > 1) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
        $model->setData($data);
        $model->save();
    }
    public function exportCustomers($custArr)
    {
        $success=0;
        $orgType = $this->_helperData->getConfigValue('general', 'orgtype');
        $requestArr = $mapping_data = $contact_mapping = [];
        $sucess=0;
        try {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $customerCollection = $objectManager->create(\Magento\Customer\Model\ResourceModel\Customer\Collection::class);
            $collection = $customerCollection->addFieldToFilter('entity_id', ['in' => $custArr]);
            $createindividualaccounts = $this->_helperData->getConfigValue('accCon', 'createindividualaccounts');
            foreach ($collection as $customer) {
                /*$customer = $cust->load($cust['entity_id']);*/
                $accSobject = $this->syncCustomerAsAccount($customer);


                $email = $customer->getEmail();



                if ($createindividualaccounts) {
                    $requestArr[] = [
                        'method' => 'PATCH',
                        'referenceId' => 'acc_' . $customer->getId(),
                        'url' => '/services/data/v46.0/sobjects/Account/webkul_es_mage__Magento_Customer_ID__c/' . $customer->getId(),
                        'body' => $accSobject
                    ];
                    if ($orgType) {
                        if ($customer->getAddresses()) {
                            foreach ($customer->getAddresses() as $address) {
                                $contactSobject = $this->syncAddressAsContact($customer, $address);
                                $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                                $requestArr[] = [
                                    'method' => 'PATCH',
                                    'referenceId' => 'cont_' . $addressId . '|' . $customer->getId(),
                                    'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                    'body' => $contactSobject
                                ];
                                break;
                            }
                        }
                    }
                } else {
                    $sfmainaccount = $this->_helperData->getConfigValue('accCon', 'sfmainaccount');
                    if ($sfmainaccount) {
                        $sfmainaccount = $this->_helperData->getConfigValue('accCon', 'sfmainaccountdetail');
                        $sfmainaccountId = $this->_json->jsonDecode($sfmainaccount)['id'];
                        $data = [
                            'magento_id' => $customer->getId(),
                            'sforce_id' => $sfmainaccountId,
                            'created_by' => $this->_helperData->getCurrentUser(),
                            'sync_status' => 1,
                            'error' => ''
                        ];
                        $this->customerMapping($data);
                        $success+=1;
                        if ($orgType) {
                            if ($customer->getAddresses()) {
                                foreach ($customer->getAddresses() as $address) {
                                    $contactSobject = $this->syncAddressAsContact($customer, $address);
                                    $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();

                                    $contactSobject->AccountId = $sfmainaccountId;
                                    $requestArr[] = [
                                        'method' => 'PATCH',
                                        'referenceId' => 'cont_' . $addressId . '|' . $customer->getId(),
                                        'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                        'body' => $contactSobject
                                    ];
                                    break;
                                }
                            }
                        }
                    } else {
                        $accountDetail = $this->_helperConnection->getSObject("SELECT AccountId,Id,webkul_es_mage__Magento_Contact_ID__c FROM Contact WHERE Email='$email'", 1, 1);

                        if (count($accountDetail) > 0) {

                            $accountId = $accountDetail['AccountId'];
                            if ($accountId) {
                                $accountObjectDetail = $this->_helperConnection->getSObject("SELECT Id,webkul_es_mage__Magento_Customer_ID__c FROM Account WHERE id='$accountId'", 1, 1);
                                if ($accountObjectDetail['webkul_es_mage__Magento_Customer_ID__c'] == $customer->getId()) {

                                    $data = [
                                        'magento_id' => $customer->getId(),
                                        'sforce_id' => $accountId,
                                        'created_by' => $this->_helperData->getCurrentUser(),
                                        'sync_status' => 1,
                                        'error' => ''
                                    ];
                                    $this->customerMapping($data);
                                    $success+=1;
                                    if ($orgType) {
                                        if ($customer->getAddresses()) {
                                            foreach ($customer->getAddresses() as $address) {
                                                $contactSobject = $this->syncAddressAsContact($customer, $address);
                                                $contactSobject->AccountId = $accountId;
                                                $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                                                $requestArr[] = [
                                                    'method' => 'PATCH',
                                                    'referenceId' => 'cont_' . $addressId . '|' . $customer->getId(),
                                                    'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                                    'body' => $contactSobject
                                                ];
                                                break;
                                            }
                                        }
                                    }
                                } else {

                                    //update Account
                                    $updateaccountdetails = $this->_helperData->getConfigValue('accCon', 'updateaccountdetails');
                                    if ($updateaccountdetails) {

                                        $accSobject->webkul_es_mage__Magento_Customer_ID__c = $customer->getId();
                                        $resp = $this->_helperConnection->updateSObject('Account', $accSobject, $accountId);
                                        $data = [
                                            'magento_id' => $customer->getId(),
                                            'sforce_id' => $resp['id'],
                                            'created_by' => $this->_helperData->getCurrentUser(),
                                            'sync_status' => 1,
                                            'error' => ''
                                        ];
                                        $this->customerMapping($data);
                                        $success+=1;
                                    } else {
                                        $newAccSObject = new \stdclass();
                                        $newAccSObject->webkul_es_mage__Magento_Customer_ID__c = $customer->getId();
                                        $resp = $this->_helperConnection->updateSObject('Account', $newAccSObject, $accountId);

                                        $data = [
                                            'magento_id' => $customer->getId(),
                                            'sforce_id' => $resp['id'],
                                            'created_by' => $this->_helperData->getCurrentUser(),
                                            'sync_status' => 1,
                                            'error' => ''
                                        ];
                                        $this->customerMapping($data);
                                        $success+=1;
                                    }
                                    if ($orgType) {
                                        if ($customer->getAddresses()) {
                                            foreach ($customer->getAddresses() as $address) {
                                                $contactSobject = $this->syncAddressAsContact($customer, $address);
                                                $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();

                                                $contactSobject->AccountId = $resp['id'];

                                                $requestArr[] = [
                                                    'method' => 'PATCH',
                                                    'referenceId' => 'cont_' . $addressId . '|' . $customer->getId(),
                                                    'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                                    'body' => $contactSobject
                                                ];
                                                break;
                                            }
                                        }
                                    }
                                }
                            } else {
                                $requestArr[] = [
                                    'method' => 'PATCH',
                                    'referenceId' => 'acc_' . $customer->getId(),
                                    'url' => '/services/data/v46.0/sobjects/Account/webkul_es_mage__Magento_Customer_ID__c/' . $customer->getId(),
                                    'body' => $accSobject
                                ];
                                if ($orgType) {
                                    if ($customer->getAddresses()) {
                                        foreach ($customer->getAddresses() as $address) {
                                            $contactSobject = $this->syncAddressAsContact($customer, $address);
                                            $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                                            $requestArr[] = [
                                                'method' => 'PATCH',
                                                'referenceId' => 'cont_' . $addressId . '|' . $customer->getId(),
                                                'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                                'body' => $contactSobject
                                            ];
                                            break;
                                        }
                                    }
                                }
                            }
                        } else {
                            $requestArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'acc_' . $customer->getId(),
                                'url' => '/services/data/v46.0/sobjects/Account/webkul_es_mage__Magento_Customer_ID__c/' . $customer->getId(),
                                'body' => $accSobject
                            ];
                            if ($orgType) {
                                if ($customer->getAddresses()) {
                                    foreach ($customer->getAddresses() as $address) {
                                        $contactSobject = $this->syncAddressAsContact($customer, $address);
                                        $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                                        $requestArr[] = [
                                            'method' => 'PATCH',
                                            'referenceId' => 'cont_' . $addressId . '|' . $customer->getId(),
                                            'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                            'body' => $contactSobject
                                        ];
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }




            }


            $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $requestArr]);

            $skipped = $errored = $succeded = 0;
            if (isset($response) && isset($response['compositeResponse'])) {
                foreach ($response['compositeResponse'] as $k => $data) {
                    if (isset($data['body']['success'])) {
                        if (strpos($data['referenceId'], 'acc_') !== false) {
                            $map = [
                                'magento_id' => ltrim($data['referenceId'], 'acc_'),
                                'sforce_id' => $data['body']['id'],
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 1,
                                'error' => ''
                            ];
                            if ($data['body']['created']) {
                                $mapping_data[] = $map;
                            } else {
                                $this->customerMapping($map);
                                $success+=1;
                            }
                            $customerId = ltrim($data['referenceId'], 'acc_');
                        } elseif (strpos($data['referenceId'], 'cont_') !== false) {
                            $ref = ltrim($data['referenceId'], 'cont_');
                            $refArr = explode('|', $ref);
                            $map = [
                                'customer_id' => $refArr[1],
                                'magento_id' => $refArr[0],
                                'sforce_id' => $data['body']['id'],
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 1,
                                'error' => ''
                            ];
                            if ($data['body']['created']) {
                                $contact_mapping[] = $map;
                            } else {
                                $this->contactMapping($map);
                            }
                        }
                        //$succeded++;
                    } elseif (!isset($data['body']['success']) && isset($data['body'][0]['errorCode'])) {
                        if (strpos($data['referenceId'], 'acc_') !== false) {
                            $map = [
                                'magento_id' => ltrim($data['referenceId'], 'acc_'),
                                'sforce_id' => '',
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 0,
                                'error' => $data['body'][0]['message']
                            ];
                            $customerId = ltrim($data['referenceId'], 'acc_');
                            $this->customerMapping($map);
                        } elseif (strpos($data['referenceId'], 'cont_') !== false) {
                            $ref = ltrim($data['referenceId'], 'cont_');
                            $refArr = explode('|', $ref);
                            $map = [
                                'customer_id' => $refArr[1],
                                'magento_id' => $refArr[0],
                                'sforce_id' => '',
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 0,
                                'error' => $data['body'][0]['message']
                            ];
                            $this->contactMapping($map);
                        }
                        $errored++;
                    } else {
                        $skipped++;
                    }
                }
            }
            if (count($mapping_data) > 0) {
                $this->_helperData->bulkInsert('wk_salesforce_eshopsync_customer_mapping', $mapping_data);
            }
            if (count($contact_mapping) > 0) {
                $this->_helperData->bulkInsert('wk_salesforce_eshopsync_contact_mapping', $contact_mapping);
            }

            return ['skip' => $skipped, 'err' => $errored, 'success' =>  $success];
        } catch (\Exception $e) {
            return ['skip' => 0, 'err' => 0, 'success' =>  0];
            $this->_helperData->eshopsyncLog($e->getMessage());
        }
    }
    public function syncCustomerAsAccount($customer)
    {
        $orgType = $this->_helperData->getConfigValue('general', 'orgtype');
        $sObject = new \stdclass();
        if ($orgType) {
            $sObject->Name = $customer->getName();
        }
        // $sObject->webkul_es_mage__Magento_Customer_ID__c = $customerId;
        $sObject->webkul_es_mage__TAX_VAT_Number__c = $customer->getTaxvat();

        if ($customer->getDefaultBilling()) {
            $billing_address_id = $customer->getDefaultBilling();
            $billing_address = $this->_customerAddress->create()->load($billing_address_id);
            if($billing_address->getcompany())
            {
                $sObject->Name = $billing_address->getcompany();
            }
            $sObject->Phone = $billing_address->getTelephone();
            $sObject->Fax = $billing_address->getFax();
            $streets = implode(',', $billing_address->getStreet());
            $sObject->BillingStreet = $streets;
            $sObject->BillingCity = $billing_address->getCity();
            $sObject->BillingPostalCode = $billing_address->getPostcode();
            $sObject->BillingState = $billing_address->getRegion();
            $sObject->BillingCountry = $billing_address->getCountryId();

            if (!$orgType) {
                $sObject->PersonMailingCity = $sObject->BillingCity;
                $sObject->PersonMailingCountry = $sObject->BillingCountry;
                $sObject->PersonMailingPostalCode = $sObject->BillingPostalCode;
                $sObject->PersonMailingState = $sObject->BillingState;
                $sObject->PersonMailingStreet = $sObject->BillingStreet;
                $sObject->PersonMobilePhone = $billing_address->getTelephone();
                // $sObject->MiddleName = $billing_address->getMiddlename();
            }
        }

        if ($customer->getDefaultShipping()) {
            $shipping_address_id = $customer->getDefaultShipping();
            $shipping_address = $this->_customerAddress->create()->load($shipping_address_id);
            $streets = implode(',', $shipping_address->getStreet());
            $sObject->ShippingStreet = $streets;
            $sObject->ShippingCity = $shipping_address->getCity();
            $sObject->ShippingPostalCode = $shipping_address->getPostcode();
            $sObject->ShippingState = $shipping_address->getRegion();
            $sObject->ShippingCountry = $shipping_address->getCountryId();
            // if(!$orgType && (!isset($sObject->MiddleName) || empty($sObject->MiddleName))){
            //     $sObject->MiddleName = $shipping_address->getMiddlename();
            // }
        }
        // if org is person type we wont need to sync address as contact in this case if user has a address but a defualt one set that in billing and shipping
        if (!$orgType && !$customer->getDefaultBilling() && $customer->getAddresses()) {
            foreach ($customer->getAddresses() as $address) {
                $sObject->Phone = $address->getTelephone();
                $sObject->Fax = $address->getFax();
                $streets = implode(',', $address->getStreet());
                $sObject->BillingStreet = $streets;
                $sObject->BillingCity = $address->getCity();
                $sObject->BillingPostalCode = $address->getPostcode();
                $sObject->BillingState = $address->getRegion();
                $sObject->BillingCountry = $address->getCountryId();

                $sObject->PersonMailingCity = $sObject->BillingCity;
                $sObject->PersonMailingCountry = $sObject->BillingCountry;
                $sObject->PersonMailingPostalCode = $sObject->BillingPostalCode;
                $sObject->PersonMailingState = $sObject->BillingState;
                $sObject->PersonMailingStreet = $sObject->BillingStreet;
                $sObject->PersonMobilePhone = $address->getTelephone();
                // $sObject->MiddleName = $address->getMiddlename();
                break;
            }
        }
        if (!$orgType && !$customer->getDefaultShipping() && $customer->getAddresses()) {
            foreach ($customer->getAddresses() as $address) {
                $streets = implode(',', $address->getStreet());
                $sObject->ShippingStreet = $streets;
                $sObject->ShippingCity = $address->getCity();
                $sObject->ShippingPostalCode = $address->getPostcode();
                $sObject->ShippingState = $address->getRegion();
                $sObject->ShippingCountry = $address->getCountryId();
                // if((!isset($sObject->MiddleName) || empty($sObject->MiddleName))){
                //     $sObject->MiddleName = $address->getMiddlename();
                // }
                break;
            }
        }
        if ($this->_helperData->getConfigValue('accCon', 'recordAcc')) {
            $recordAccount = $this->_helperData->getConfigValue('accCon', 'recordAcc');
            $sObject->RecordTypeId = $this->_json->jsonDecode($recordAccount)['id'];
        }
        // Fields for person account
        if (!$orgType) {
            $custData = $customer->getData();
            if (isset($custData['lastname'])) {
                $lastName = strlen($custData['lastname']) > 40 ? mb_strimwidth($custData['lastname'], 0, 37, '...') : $custData['lastname'];
            }
            $sObject->FirstName = strlen($customer->getFirstname()) > 40 ? mb_strimwidth($customer->getFirstname(), 0, 37, "...") : $customer->getFirstname();
            if ($customer->getLastName()) {
                $sObject->LastName = strlen($customer->getLastName()) > 40 ? mb_strimwidth($customer->getLastName(), 0, 37, "...") : $customer->getLastName();
            } elseif ($lastName) {
                $sObject->LastName = $lastName;
            } else {
                $sObject->LastName = isset($billing_address) ? $billing_address->getFirstname() : $customer->getEmail();
            }

            $sObject->PersonEmail = $customer->getEmail();
        }


        $accFieldMap = $this->_helperData->getConfigValue('acc', 'accountFieldMap');
        if ($accFieldMap) {
            $accFieldMap = $this->_json->jsonDecode($accFieldMap);
            $customerData = $customer->getData();
            if(is_array($accFieldMap) && count($accFieldMap) > 0){
                foreach ($accFieldMap as $field) {

                    $magentoField = $field['magentoField'];
                    $sfField = $field['saleForceField'];
                    $sObject->$sfField = $customerData[$magentoField];
                }
            }
        }
        return $sObject;
    }
    public function syncAddressAsContact($customer, $address)
    {
        $sObject = new \stdclass();
        if ($customer->getDefaultBilling()) {
            $billing_address_id = $customer->getDefaultBilling();
            $billing_address = $this->_customerAddress->create()->load($billing_address_id);
            $streets = implode(',', $billing_address->getStreet());
            $sObject->MailingStreet = $streets;
            $sObject->MailingCity = $billing_address->getCity();
            $sObject->MailingPostalCode = $billing_address->getPostcode();
            $sObject->MailingCountry = $billing_address->getCountryId();
            $sObject->MailingState = $billing_address->getRegion();
        }
        else{
            $streets = implode(',', $address->getStreet());
            $sObject->MailingStreet = $streets;
            $sObject->MailingCity = $address->getCity();
            $sObject->MailingPostalCode = $address->getPostcode();
            $sObject->MailingCountry = $address->getCountryId();
            $sObject->MailingState = $address->getRegion();
        }
        if($customer->getDefaultShipping()) {
            $shipping_address_id = $customer->getDefaultShipping();
            $shipping_address = $this->_customerAddress->create()->load($shipping_address_id);
            $streets = implode(',', $shipping_address->getStreet());
            $sObject->OtherStreet = $streets;
            $sObject->OtherCity = $shipping_address->getCity();
            $sObject->OtherPostalCode = $shipping_address->getPostcode();
            $sObject->OtherState = $shipping_address->getRegion();
            $sObject->OtherCountry = $shipping_address->getCountryId();
        }
        $addressId = $address->getId();

        $customerId = $customer->getId();

        $sObject->FirstName = $address->getFirstname();
        $sObject->LastName = $address->getLastname();
        $sObject->AccountId = '@{acc_' . $customerId . '.id}';
        $sObject->Email = $customer->getEmail();
        $sObject->Phone = $address->getTelephone();
        if ($this->_helperData->getConfigValue('accCon', 'recordCon')) {
            $recordContact = $this->_helperData->getConfigValue('accCon', 'recordCon');
            $sObject->RecordTypeId = $this->_json->jsonDecode($recordContact)['id'];
        }
        // $sObject->webkul_es_mage__Magento_Contact_ID__c = $addressId;
        return $sObject;
    }
    public function exportSpecificCustomer($customerId)
    {
        $requestArr = $mapping_data = $contact_mapping = [];
        $sfId = 0;
        $orgType = $this->_helperData->getConfigValue('general', 'orgtype');
        $createindividualaccounts = $this->_helperData->getConfigValue('accCon', 'createindividualaccounts');
        try {
            $customer = $this->_customerCustomer->create()->load($customerId);
            $accSobject = $this->syncCustomerAsAccount($customer);

            $email = $customer->getEmail();

            if ($createindividualaccounts || (!$orgType)) {

                $requestArr[] = [
                    'method' => 'PATCH',
                    'referenceId' => 'acc_' . $customerId,
                    'url' => '/services/data/v46.0/sobjects/Account/webkul_es_mage__Magento_Customer_ID__c/' . $customerId,
                    'body' => $accSobject
                ];
                if ($orgType) {
                    if ($customer->getAddresses()) {
                        foreach ($customer->getAddresses() as $address) {
                            $contactSobject = $this->syncAddressAsContact($customer, $address);
                            $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                            $requestArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'cont_' . $addressId . '|' . $customerId,
                                'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                'body' => $contactSobject
                            ];
                            break;
                        }
                    }
                }
            } else {
                $sfmainaccount = $this->_helperData->getConfigValue('accCon', 'sfmainaccount');
                if ($sfmainaccount) {
                    $sfmainaccount = $this->_helperData->getConfigValue('accCon', 'sfmainaccountdetail');
                    $sfmainaccountId = $this->_json->jsonDecode($sfmainaccount)['id'];
                    $data = [
                        'magento_id' => $customerId,
                        'sforce_id' => $sfmainaccountId,
                        'created_by' => $this->_helperData->getCurrentUser(),
                        'sync_status' => 1,
                        'error' => ''
                    ];
                    $this->customerMapping($data);
                    if ($orgType) {
                        if ($customer->getAddresses()) {
                            foreach ($customer->getAddresses() as $address) {
                                $contactSobject = $this->syncAddressAsContact($customer, $address);
                                $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();

                                $contactSobject->AccountId = $sfmainaccountId;
                                $requestArr[] = [
                                    'method' => 'PATCH',
                                    'referenceId' => 'cont_' . $addressId . '|' . $customerId,
                                    'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                    'body' => $contactSobject
                                ];
                                break;
                            }
                        }
                    }
                } else {

                    $accountDetail = $this->_helperConnection->getSObject("SELECT AccountId,Id,webkul_es_mage__Magento_Contact_ID__c FROM Contact WHERE Email='$email'", 1, 1);

                    if (count($accountDetail) > 0) {

                        $accountId = $accountDetail['AccountId'];
                        if ($accountId) {
                            $accountObjectDetail = $this->_helperConnection->getSObject("SELECT Id,webkul_es_mage__Magento_Customer_ID__c FROM Account WHERE id='$accountId'", 1, 1);
                            if ($accountObjectDetail['webkul_es_mage__Magento_Customer_ID__c'] == $customerId) {

                                $data = [
                                    'magento_id' => $customerId,
                                    'sforce_id' => $accountId,
                                    'created_by' => $this->_helperData->getCurrentUser(),
                                    'sync_status' => 1,
                                    'error' => ''
                                ];
                                $this->customerMapping($data);

                                if ($orgType) {
                                    if ($customer->getAddresses()) {
                                        foreach ($customer->getAddresses() as $address) {
                                            $contactSobject = $this->syncAddressAsContact($customer, $address);
                                            $contactSobject->AccountId = $accountId;
                                            $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                                            $requestArr[] = [
                                                'method' => 'PATCH',
                                                'referenceId' => 'cont_' . $addressId . '|' . $customerId,
                                                'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                                'body' => $contactSobject
                                            ];
                                            break;
                                        }
                                    }
                                }
                            } else {

                                //update Account
                                $updateaccountdetails = $this->_helperData->getConfigValue('accCon', 'updateaccountdetails');
                                if ($updateaccountdetails) {

                                    $accSobject->webkul_es_mage__Magento_Customer_ID__c = $customerId;
                                    $resp = $this->_helperConnection->updateSObject('Account', $accSobject, $accountId);
                                    $data = [
                                        'magento_id' => $customerId,
                                        'sforce_id' => $resp['id'],
                                        'created_by' => $this->_helperData->getCurrentUser(),
                                        'sync_status' => 1,
                                        'error' => ''
                                    ];
                                    $this->customerMapping($data);
                                } else {
                                    $newAccSObject = new \stdclass();
                                    $newAccSObject->webkul_es_mage__Magento_Customer_ID__c = $customerId;
                                    $resp = $this->_helperConnection->updateSObject('Account', $newAccSObject, $accountId);

                                    $data = [
                                        'magento_id' => $customerId,
                                        'sforce_id' => $resp['id'],
                                        'created_by' => $this->_helperData->getCurrentUser(),
                                        'sync_status' => 1,
                                        'error' => ''
                                    ];
                                    $this->customerMapping($data);
                                }
                                if ($orgType) {
                                    if ($customer->getAddresses()) {
                                        foreach ($customer->getAddresses() as $address) {
                                            $contactSobject = $this->syncAddressAsContact($customer, $address);
                                            $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();

                                            $contactSobject->AccountId = $resp['id'];

                                            $requestArr[] = [
                                                'method' => 'PATCH',
                                                'referenceId' => 'cont_' . $addressId . '|' . $customerId,
                                                'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                                'body' => $contactSobject
                                            ];
                                            break;
                                        }
                                    }
                                }
                            }
                        } else {
                            $requestArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'acc_' . $customerId,
                                'url' => '/services/data/v46.0/sobjects/Account/webkul_es_mage__Magento_Customer_ID__c/' . $customerId,
                                'body' => $accSobject
                            ];
                            if ($orgType) {
                                if ($customer->getAddresses()) {
                                    foreach ($customer->getAddresses() as $address) {
                                        $contactSobject = $this->syncAddressAsContact($customer, $address);
                                        $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                                        $requestArr[] = [
                                            'method' => 'PATCH',
                                            'referenceId' => 'cont_' . $addressId . '|' . $customerId,
                                            'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                            'body' => $contactSobject
                                        ];
                                        break;
                                    }
                                }
                            }
                        }
                    } else {
                        $requestArr[] = [
                            'method' => 'PATCH',
                            'referenceId' => 'acc_' . $customerId,
                            'url' => '/services/data/v46.0/sobjects/Account/webkul_es_mage__Magento_Customer_ID__c/' . $customerId,
                            'body' => $accSobject
                        ];
                        if ($orgType) {
                            if ($customer->getAddresses()) {
                                foreach ($customer->getAddresses() as $address) {
                                    $contactSobject = $this->syncAddressAsContact($customer, $address);
                                    $addressId = $customer->getDefaultBilling() ? $customer->getDefaultBilling() : $address->getId();


                                    $requestArr[] = [
                                        'method' => 'PATCH',
                                        'referenceId' => 'cont_' . $addressId . '|' . $customerId,
                                        'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $addressId,
                                        'body' => $contactSobject
                                    ];
                                    break;
                                }
                            }
                        }
                    }
                }
            }


            $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $requestArr]);

            if (isset($response) && isset($response['compositeResponse'])) {
                foreach ($response['compositeResponse'] as $k => $data) {
                    if (isset($data['body']['success'])) {
                        if (strpos($data['referenceId'], 'acc_') !== false) {
                            $sfId = $data['body']['id'];
                            $customerId = ltrim($data['referenceId'], 'acc_');
                            $mapping_data = [
                                'magento_id' => $customerId,
                                'sforce_id' => $data['body']['id'],
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 1,
                                'error' => ''
                            ];
                            $this->customerMapping($mapping_data);
                        } elseif (strpos($data['referenceId'], 'cont_') !== false) {
                            $ref = ltrim($data['referenceId'], 'cont_');
                            $refArr = explode('|', $ref);
                            $map = [
                                'customer_id' => $refArr[1],
                                'magento_id' => $refArr[0],
                                'sforce_id' => $data['body']['id'],
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 1,
                                'error' => ''
                            ];
                            if ($data['body']['created']) {
                                $contact_mapping[] = $map;
                            } else {
                                $this->contactMapping($map);
                            }
                        }
                        //$succeded = 1;
                    } elseif (!isset($data['body']['success']) && isset($data['body'][0]['errorCode'])) {
                        if (strpos($data['referenceId'], 'acc_') !== false) {
                            $customerId = ltrim($data['referenceId'], 'acc_');
                            $mapping_data = [
                                'magento_id' => $customerId,
                                'sforce_id' => '',
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 0,
                                'error' => $data['body'][0]['message']
                            ];
                            $this->customerMapping($mapping_data);
                        } elseif (strpos($data['referenceId'], 'cont_') !== false) {
                            $ref = ltrim($data['referenceId'], 'cont_');
                            $refArr = explode('|', $ref);
                            $map = [
                                'customer_id' => $refArr[1],
                                'magento_id' => $refArr[0],
                                'sforce_id' => '',
                                'created_by' => $this->_helperData->getCurrentUser(),
                                'sync_status' => 0,
                                'error' => $data['body'][0]['message']
                            ];
                            $this->contactMapping($map);
                        }
                    }
                }
            }
            if (count($contact_mapping) > 0) {
                $this->_helperData->bulkInsert('wk_salesforce_eshopsync_contact_mapping', $contact_mapping);
            }
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
        }
        return $sfId;
    }
    public function deleteMapping($customerId)
    {
        $model = $this->_eshopsyncCustomer->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $customerId)->getData();
        if (count($collection) > 0) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
    }
    public function deleteContactMapping($contactId)
    {
        $model = $this->_eshopsyncContact->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $contactId)->getData();
        if (count($collection) > 0) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
    }
    public function getExisitingAccountByEmail($email)
    {
        // getaccount from SF by email
        $query = "SELECT Id from Account WHERE (PersonEmail='$email')";
        $account = $this->_helperConnection->getSObject($query);
        $accountId = false;
        if (isset($account['Id']) && !empty($account['Id'])) {
            $accountId = $account['Id'];
        }
        return $accountId;
    }
}
